==> nono-0.7.3.tar.gz: nono-0.7.3/vm/mpu.cpp <==
//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// MPU (共通部分)
//

// Device
//    |  +-----------+
//    +--| MPUDevice | (全 MPU の共通部)
//       +-----------+
//         |
//         |  +---------------+
//         +--| MainMPUDevice | (680x0/88xx0 の共通部)
//         |  +---------------+
//         |    |
//         |    +-- MPU680x0Device
//         |    +-- MPU88xx0Device
//         |
//         +-- MPU64180Device

#include "mpu.h"
#include "config.h"
#include "debugger.h"
#include "mainbus.h"
#include "scheduler.h"
#include "syncer.h"

//
// 全 MPU 共通
//

// コンストラクタ
MPUDevice::MPUDevice(uint objid_)
	: inherited(objid_)
{
}

// デストラクタ
MPUDevice::~MPUDevice()
{
}

// 初期化
bool
MPUDevice::Init()
{
	if (inherited::Init() == false) {
		return false;
	}

	debugger = GetDebugger();

	// パラメータは継承側でセットしている。
	scheduler->RegistEvent(exec_event);

	return true;
}

void
MPUDevice::PowerOff()
{
	scheduler->StopEvent(exec_event);
}


//
// メイン MPU (680x0/88xx0) 共通部
//

// コンストラクタ
MainMPUDevice::MainMPUDevice()
	: inherited(OBJ_MPU)
{
}

// デストラクタ
MainMPUDevice::~MainMPUDevice()
{
}

// 初期化
bool
MainMPUDevice::Init()
{
	if (inherited::Init() == false) {
		return false;
	}

	mainbus = GetMainbusDevice();
	syncer = GetSyncer();

	// MPU クロック
	const ConfigItem& item = gConfig->Find("mpu-clock");
	const std::string& val = item.AsString();
	if (val.empty()) {
		item.Err();
		return false;
	}
	// 文字列を double に変換
	char *end;
	errno = 0;
	double f = strtod(val.c_str(), &end);
	if (end == val.c_str() || end[0] != '\0' || errno == ERANGE) {
		item.Err();
		return false;
	}
	// 設定の "mpu-clock" は MHz 単位だが、変数 clock_khz は kHz 単位。
	clock_khz = (uint)(f * 1000);
	// 1MHz 未満はエラー。実際 10MHz 未満でもいい気がするけど
	if (clock_khz < 1000) {
		item.Err();
		return false;
	}
	clock_nsec = 1000 * 1000 / clock_khz;

	scheduler->ConnectMessage(MessageID::MPU_TRACE_MAIN, this,
		ToMessageCallback(&MainMPUDevice::TraceMessage));

	exec_event.SetName("MPU Execute");

	return true;
}

// MPU トレース状態設定要求メッセージ
void
MainMPUDevice::TraceMessage(MessageID msgid, uint32 arg)
{
	// リセット中は SetTrace しない
	if (resetting) {
		return;
	}

	// デバッガから MPU のトレース状態を設定してくれと言われた
	SetTrace((bool)arg);
}

// Round Mode (モニタ表示用)
/*static*/ const char * const
MainMPUDevice::rmstr[4] = {
	"Near",
	"Zero",
	"Minus",
	"Plus",
};

==> nono-1.0.0.tar.gz: nono-1.0.0/vm/mpu.cpp <==
//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// MPU (共通部分)
//

// Device
//    |  +-----------+
//    +--| MPUDevice | (全 MPU の共通部)
//       +-----------+
//         |
//         |  +---------------+
//         +--| MainMPUDevice | (680x0/88xx0 の共通部)
//         |  +---------------+
//         |    |
//         |    +-- MPU680x0Device
//         |    +-- MPU88xx0Device
//         |
//         +-- MPU64180Device

#include "mpu.h"
#include "config.h"
#include "debugger.h"
#include "mainbus.h"
#include "scheduler.h"
#include "syncer.h"

//
// 全 MPU 共通
//

// コンストラクタ
MPUDevice::MPUDevice(uint objid_)
	: inherited(objid_)
{
}

// デストラクタ
MPUDevice::~MPUDevice()
{
}

// 初期化
bool
MPUDevice::Init()
{
	if (inherited::Init() == false) {
		return false;
	}

	debugger = GetDebugger();

	// パラメータは継承側でセットしている。
	scheduler->RegistEvent(exec_event);

	return true;
}

void
MPUDevice::PowerOff()
{
	scheduler->StopEvent(exec_event);
}


//
// メイン MPU (680x0/88xx0) 共通部
//

// コンストラクタ
MainMPUDevice::MainMPUDevice()
	: inherited(OBJ_MPU)
{
}

// デストラクタ
MainMPUDevice::~MainMPUDevice()
{
}

// 初期化
bool
MainMPUDevice::Init()
{
	if (inherited::Init() == false) {
		return false;
	}

	mainbus = GetMainbusDevice();
	syncer = GetSyncer();

	// MPU クロック
	const ConfigItem& item = gConfig->Find("mpu-clock");

#if 1
	int val;
	if (item.TryFixedDecimal(&val, 3) == false) {
		item.Err();
		return false;
	}
	clock_khz = (uint)val;
#else
	const std::string& val = item.AsString();
	if (val.empty()) {
		item.Err();
		return false;
	}
	// 文字列を double に変換
	char *end;
	errno = 0;
	double f = strtod(val.c_str(), &end);
	if (end == val.c_str() || end[0] != '\0' || errno == ERANGE) {
		item.Err();
		return false;
	}
	// 設定の "mpu-clock" は MHz 単位だが、変数 clock_khz は kHz 単位。
	clock_khz = (uint)(f * 1000 + 0.9);
#endif
	// 1MHz 未満はエラー。実際 10MHz 未満でもいい気がするけど
	if (clock_khz < 1000) {
		item.Err();
		return false;
	}
	clock_nsec = 1000 * 1000 / clock_khz;

	scheduler->ConnectMessage(MessageID::MPU_TRACE_MAIN, this,
		ToMessageCallback(&MainMPUDevice::TraceMessage));

	exec_event.SetName("MPU Execute");

	return true;
}

// MPU トレース状態設定要求メッセージ
void
MainMPUDevice::TraceMessage(MessageID msgid, uint32 arg)
{
	// リセット中は SetTrace しない
	if (resetting) {
		return;
	}

	// デバッガから MPU のトレース状態を設定してくれと言われた
	SetTrace((bool)arg);
}

// Round Mode (モニタ表示用)
/*static*/ const char * const
MainMPUDevice::rmstr[4] = {
	"Near",
	"Zero",
	"Minus",
	"Plus",
};
